@@ -1,13 +1,14 @@
 import datetime
 from re import match
 from time import sleep
-from typing import Union
+from typing import Optional, Union
 
 from mnamer.exceptions import (
     MnamerException,
     MnamerNetworkException,
     MnamerNotFoundException,
 )
+from mnamer.language import Language
 from mnamer.utils import clean_dict, parse_date, request_json
 
 __all__ = [
@@ -32,44 +33,19 @@ __all__ = [
     "tvmaze_show_single_search",
 ]
 
-TVDB_LANGUAGE_CODES = [
-    "cs",
-    "da",
-    "de",
-    "el",
-    "en",
-    "es",
-    "fi",
-    "fr",
-    "he",
-    "hr",
-    "hu",
-    "it",
-    "ja",
-    "ko",
-    "nl",
-    "no",
-    "pl",
-    "pt",
-    "ru",
-    "sl",
-    "sv",
-    "tr",
-    "zh",
-]
 OMDB_PLOT_TYPES = {"short", "long"}
 MAX_RETRIES = 5
 
 
 def omdb_title(
     api_key: str,
-    id_imdb: str = None,
-    media: str = None,
-    title: str = None,
-    season: int = None,
-    episode: int = None,
-    year: int = None,
-    plot: str = None,
+    id_imdb: Optional[str] = None,
+    media: Optional[str] = None,
+    title: Optional[str] = None,
+    season: Optional[int] = None,
+    episode: Optional[int] = None,
+    year: Optional[int] = None,
+    plot: Optional[str] = None,
     cache: bool = True,
 ) -> dict:
     """
@@ -99,7 +75,7 @@ def omdb_title(
     error = content.get("Error") if isinstance(content, dict) else None
     if status == 401:
         raise MnamerException("invalid API key")
-    elif status != 200 or not isinstance(content, dict):
+    elif status != 200 or not isinstance(content, dict):  # pragma: no cover
         raise MnamerNetworkException("OMDb down or unavailable?")
     elif error:
         raise MnamerNotFoundException(error)
@@ -109,8 +85,8 @@ def omdb_title(
 def omdb_search(
     api_key: str,
     query: str,
-    year: int = None,
-    media: str = None,
+    year: Optional[int] = None,
+    media: Optional[str] = None,
     page: int = 1,
     cache: bool = True,
 ) -> dict:
@@ -119,7 +95,7 @@ def omdb_search(
 
     Online docs: http://www.omdbapi.com/#parameters.
     """
-    if 1 > page > 100:
+    if page < 1 or page > 100:
         raise MnamerException("page must be between 1 and 100")
     url = "http://www.omdbapi.com"
     parameters = {
@@ -144,7 +120,7 @@ def tmdb_find(
     api_key: str,
     external_source: str,
     external_id: str,
-    language: str = "en-US",
+    language: Optional[Language] = None,
     cache: bool = True,
 ) -> dict:
     """
@@ -187,7 +163,7 @@ def tmdb_find(
 def tmdb_movies(
     api_key: str,
     id_tmdb: Union[int, str],
-    language: str = "en-US",
+    language: Optional[str] = None,
     cache: bool = True,
 ) -> dict:
     """
@@ -195,10 +171,7 @@ def tmdb_movies(
 
     Online docs: developers.themoviedb.org/3/movies.
     """
-    try:
-        url = f"https://api.themoviedb.org/3/movie/{id_tmdb}"
-    except ValueError:
-        raise MnamerException("id_tmdb must be numeric")
+    url = f"https://api.themoviedb.org/3/movie/{id_tmdb}"
     parameters = {"api_key": api_key, "language": language}
     status, content = request_json(url, parameters, cache=cache)
     if status == 401:
@@ -213,9 +186,10 @@ def tmdb_movies(
 def tmdb_search_movies(
     api_key: str,
     title: str,
-    year: int = None,
+    year: Optional[Union[int, str]] = None,
+    language: Optional[Language] = None,
+    region: Optional[str] = None,
     adult: bool = False,
-    region: str = None,
     page: int = 1,
     cache: bool = True,
 ) -> dict:
@@ -225,16 +199,12 @@ def tmdb_search_movies(
     Online docs: developers.themoviedb.org/3/search/search-movies.
     """
     url = "https://api.themoviedb.org/3/search/movie"
-    try:
-        if year:
-            year = int(year)
-    except ValueError:
-        raise MnamerException("year must be numeric")
     parameters = {
         "api_key": api_key,
         "query": title,
         "page": page,
         "include_adult": adult,
+        "language": language,
         "region": region,
         "year": year,
     }
@@ -282,33 +252,40 @@ def tvdb_refresh_token(token: str) -> str:
 
 
 def tvdb_episodes_id(
-    token: str, id_tvdb: Union[str, int], lang: str = "en", cache: bool = True
+    token: str,
+    id_tvdb: Union[str, int],
+    language: Optional[Language] = None,
+    cache: bool = True,
 ) -> dict:
     """
     Returns the full information for a given episode id.
 
     Online docs: https://api.thetvdb.com/swagger#!/Episodes.
     """
-    if lang not in TVDB_LANGUAGE_CODES:
-        raise MnamerException(
-            "'lang' must be one of %s" % ",".join(TVDB_LANGUAGE_CODES)
-        )
+    Language.ensure_valid_for_tvdb(language)
     url = f"https://api.thetvdb.com/episodes/{id_tvdb}"
-    headers = {"Accept-Language": lang, "Authorization": f"Bearer {token}"}
-    status, content = request_json(url, headers=headers, cache=cache)
+    headers = {"Authorization": f"Bearer {token}"}
+    if language:
+        headers["Accept-Language"] = language.a2
+    status, content = request_json(
+        url, headers=headers, cache=cache is True and language is None
+    )
     if status == 401:
         raise MnamerException("invalid token")
     elif status == 404:
         raise MnamerNotFoundException
-    elif status == 200 and "invalidLanguage" in content.get("errors", {}):
-        raise MnamerNotFoundException
     elif status != 200 or not content.get("data"):  # pragma: no cover
         raise MnamerNetworkException("TVDb down or unavailable?")
+    elif content["data"]["id"] == 0:
+        raise MnamerNotFoundException
     return content
 
 
 def tvdb_series_id(
-    token: str, id_tvdb: Union[str, int], lang: str = "en", cache: bool = True
+    token: str,
+    id_tvdb: Union[str, int],
+    language: Optional[Language] = None,
+    cache: bool = True,
 ) -> dict:
     """
     Returns a series records that contains all information known about a
@@ -316,19 +293,22 @@ def tvdb_series_id(
 
     Online docs: api.thetvdb.com/swagger#!/Series/get_series_id.
     """
-    if lang not in TVDB_LANGUAGE_CODES:
-        raise MnamerException(
-            "'lang' must be one of %s" % ",".join(TVDB_LANGUAGE_CODES)
-        )
+    Language.ensure_valid_for_tvdb(language)
     url = f"https://api.thetvdb.com/series/{id_tvdb}"
-    headers = {"Accept-Language": lang, "Authorization": f"Bearer {token}"}
-    status, content = request_json(url, headers=headers, cache=cache)
+    headers = {"Authorization": f"Bearer {token}"}
+    if language:
+        headers["Accept-Language"] = language.a2
+    status, content = request_json(
+        url, headers=headers, cache=cache is True and language is None
+    )
     if status == 401:
         raise MnamerException("invalid token")
     elif status == 404:
         raise MnamerNotFoundException
     elif status != 200 or not content.get("data"):  # pragma: no cover
         raise MnamerNetworkException("TVDb down or unavailable?")
+    elif content["data"]["id"] == 0:
+        raise MnamerNotFoundException
     return content
 
 
@@ -336,7 +316,7 @@ def tvdb_series_id_episodes(
     token: str,
     id_tvdb: Union[str, int],
     page: int = 1,
-    lang: str = "en",
+    language: Optional[Language] = None,
     cache: bool = True,
 ) -> dict:
     """
@@ -345,15 +325,17 @@ def tvdb_series_id_episodes(
     Note: Paginated with 100 results per page.
     Online docs: api.thetvdb.com/swagger#!/Series/get_series_id_episodes.
     """
-    if lang not in TVDB_LANGUAGE_CODES:
-        raise MnamerException(
-            "'lang' must be one of %s" % ",".join(TVDB_LANGUAGE_CODES)
-        )
+    Language.ensure_valid_for_tvdb(language)
     url = f"https://api.thetvdb.com/series/{id_tvdb}/episodes"
-    headers = {"Accept-Language": lang, "Authorization": f"Bearer {token}"}
+    headers = {"Authorization": f"Bearer {token}"}
+    if language:
+        headers["Accept-Language"] = language.a2
     parameters = {"page": page}
     status, content = request_json(
-        url, parameters, headers=headers, cache=cache
+        url,
+        parameters,
+        headers=headers,
+        cache=cache is True and language is None,
     )
     if status == 401:
         raise MnamerException("invalid token")
@@ -367,10 +349,10 @@ def tvdb_series_id_episodes(
 def tvdb_series_id_episodes_query(
     token: str,
     id_tvdb: Union[str, int],
-    episode: int = None,
-    season: int = None,
+    episode: Optional[int] = None,
+    season: Optional[int] = None,
     page: int = 1,
-    lang: str = "en",
+    language: Optional[Language] = None,
     cache: bool = True,
 ) -> dict:
     """
@@ -380,15 +362,17 @@ def tvdb_series_id_episodes_query(
     ever need to query against both tvdb and imdb series ids?
     Online docs: api.thetvdb.com/swagger#!/Series/get_series_id_episodes_query.
     """
-    if lang not in TVDB_LANGUAGE_CODES:
-        raise MnamerException(
-            "'lang' must be one of %s" % ",".join(TVDB_LANGUAGE_CODES)
-        )
+    Language.ensure_valid_for_tvdb(language)
     url = f"https://api.thetvdb.com/series/{id_tvdb}/episodes/query"
-    headers = {"Accept-Language": lang, "Authorization": f"Bearer {token}"}
+    headers = {"Authorization": f"Bearer {token}"}
+    if language:
+        headers["Accept-Language"] = language.a2
     parameters = {"airedSeason": season, "airedEpisode": episode, "page": page}
     status, content = request_json(
-        url, parameters, headers=headers, cache=cache
+        url,
+        parameters,
+        headers=headers,
+        cache=cache is True and language is None,
     )
     if status == 401:
         raise MnamerException("invalid token")
@@ -401,10 +385,10 @@ def tvdb_series_id_episodes_query(
 
 def tvdb_search_series(
     token: str,
-    series: str = None,
-    id_imdb: str = None,
-    id_zap2it: str = None,
-    lang: str = "en",
+    series: Optional[str] = None,
+    id_imdb: Optional[str] = None,
+    id_zap2it: Optional[str] = None,
+    language: Optional[Language] = None,
     cache: bool = True,
 ) -> dict:
     """
@@ -413,15 +397,17 @@ def tvdb_search_series(
     Online docs: https://api.thetvdb.com/swagger#!/Search/get_search_series
     Note: results a maximum of 100 entries per page, no option for pagination.
     """
-    if lang not in TVDB_LANGUAGE_CODES:
-        raise MnamerException(
-            "'lang' must be one of %s" % ",".join(TVDB_LANGUAGE_CODES)
-        )
+    Language.ensure_valid_for_tvdb(language)
     url = "https://api.thetvdb.com/search/series"
     parameters = {"name": series, "imdbId": id_imdb, "zap2itId": id_zap2it}
-    headers = {"Accept-Language": lang, "Authorization": f"Bearer {token}"}
+    headers = {"Authorization": f"Bearer {token}"}
+    if language:
+        headers["Accept-Language"] = language.a2
     status, content = request_json(
-        url, parameters, headers=headers, cache=cache
+        url,
+        parameters,
+        headers=headers,
+        cache=cache is True and language is None,
     )
     if status == 401:
         raise MnamerException("invalid token")
@@ -473,7 +459,7 @@ def tvmaze_show_search(
 
     Online docs: https://www.tvmaze.com/api#show-search
     """
-    url = f"http://api.tvmaze.com/search/shows"
+    url = "http://api.tvmaze.com/search/shows"
     parameters = {"q": query}
     status, content = request_json(url, parameters, cache=cache)
     if status == 443 and attempt <= MAX_RETRIES:  # pragma: no cover
@@ -497,7 +483,7 @@ def tvmaze_show_single_search(
 
     Online docs: https://www.tvmaze.com/api#show-single-search
     """
-    url = f"http://api.tvmaze.com/singlesearch/shows"
+    url = "http://api.tvmaze.com/singlesearch/shows"
     parameters = {"q": query}
     status, content = request_json(url, parameters, cache=cache)
     if status == 443 and attempt <= MAX_RETRIES:  # pragma: no cover
@@ -511,9 +497,8 @@ def tvmaze_show_single_search(
 
 
 def tvmaze_show_lookup(
-    id_imdb: str = None,
+    id_imdb: Optional[str] = None,
     id_tvdb: Union[str, int] = None,
-    embed_episodes: bool = False,
     cache: bool = True,
     attempt: int = 1,
 ) -> dict:
@@ -525,16 +510,12 @@ def tvmaze_show_lookup(
     """
     if not [id_imdb, id_tvdb].count(None) == 1:
         raise MnamerException("id_imdb and id_tvdb are mutually exclusive")
-    url = f"http://api.tvmaze.com/lookup/shows"
+    url = "http://api.tvmaze.com/lookup/shows"
     parameters = {"imdb": id_imdb, "thetvdb": id_tvdb}
-    if embed_episodes:
-        parameters["embed"] = "episodes"
     status, content = request_json(url, parameters, cache=cache)
     if status == 443 and attempt <= MAX_RETRIES:  # pragma: no cover
         sleep(attempt * 2)
-        return tvmaze_show_lookup(
-            id_imdb, id_tvdb, embed_episodes, cache, attempt + 1
-        )
+        return tvmaze_show_lookup(id_imdb, id_tvdb, cache, attempt + 1)
     elif status == 404:
         raise MnamerNotFoundException
     elif status != 200 or not content:  # pragma: no cover
